/**************************************************************************/
/* tg_load.c -  Functions relating to loading info into the global Acts   */
/*                and Eps arrays, for the "Trek Guru" program.            */
/*                                                                        */
/*                Original creator: Rob Seace (ras@world.std.com)         */
/*                                                                        */
/*        Note: If you make ANY changes at all to any of the code, PLEASE */
/*              make note of it with comments, as well as in the "README" */
/*              file, especially if re-distributing the new code.         */
/**************************************************************************/


#include "trekguru.h"

extern    long    ftell ();


/* tg_load_one - Loads info from passed data files into Acts and Eps, and
                 sets Num_Acts and Num_Eps appropriately.
*/

void tg_load_one (actfile, epsfile)
    char    *actfile;            /* name of actor info file              */
    char    *epsfile;            /* name of episode info file            */
{
    FILE    *afp;                /* file pointer to actor info file      */
    FILE    *efp;                /* file pointer to episode info file    */
    char    buf[MAX_BUF_LEN+1];  /* buffer for user input                */

    if (Acts ISNT NULL) {
        free (Acts);
        Acts = NULL;
    }
    if (Eps ISNT NULL) {
        free (Eps);
        Eps = NULL;
    }

    if ((afp = fopen (actfile, "r")) IS NULL) {
        printf ("\07\nCan't open actor data file %s!\n\n", actfile);
        printf ("This file may not be available yet.  Press ENTER to continue.");
        fgets (buf, MAX_BUF_LEN, stdin);
        return;
    }

    if ((efp = fopen (epsfile, "r")) IS NULL) {
        printf ("\07\nCan't open episode data file %s!\n\n", epsfile);
        printf ("This file may not be available yet.  Press ENTER to continue.");
        fgets (buf, MAX_BUF_LEN, stdin);
        fclose (afp);
        return;
    }

    fseek (afp, 0L, SEEK_END);        /* go to end of file */
    Num_Acts = (int) (ftell (afp) / ACTREC_LEN);
    fseek (afp, 0L, SEEK_SET);        /* go back to beginning */

    fseek (efp, 0L, SEEK_END);        /* go to end of file */
    Num_Eps = (int) (ftell (efp) / EPSREC_LEN);
    fseek (efp, 0L, SEEK_SET);        /* go back to beginning */

    if ((Acts = (actrec) calloc (Num_Acts, ACTREC_LEN)) IS NULL) {
        fprintf (stderr, "\nNot enough memory for array of %d actor records (%d bytes each).\n", Num_Acts, ACTREC_LEN);
        fprintf (stderr, "\nEither wait until more memory is available, or run the program");
        fprintf (stderr, "\nwith the '+f' command line argument.\n\n");
        fclose (afp);
        fclose (efp);
        exit (-1);
    }

    if ((Eps = (epsrec) calloc (Num_Eps, EPSREC_LEN)) IS NULL) {
        fprintf (stderr, "\nNot enough memory for array of %d episode records (%d bytes each).\n", Num_Eps, EPSREC_LEN);
        fprintf (stderr, "\nEither wait until more memory is available, or run the program");
        fprintf (stderr, "\nwith the '+f' command line argument.\n\n");
        free (Acts);
        fclose (afp);
        fclose (efp);
        exit (-1);
    }

    tg_fill_eps (efp, 0, Num_Eps);
    tg_fill_acts (afp, 0, Num_Acts);

    fclose (afp);
    fclose (efp);
}


/* tg_load_all - Loads info from all of the available data files into global
                 Acts and Eps arrays, and sets Num_Acts and Num_Eps.
*/

void tg_load_all ()
{
    FILE    *tos_afp;          /* file pointer to TOS actor info file      */
    FILE    *tas_afp;          /* file pointer to TAS actor info file      */
    FILE    *tms_afp;          /* file pointer to TMS actor info file      */
    FILE    *tng_afp;          /* file pointer to TNG actor info file      */
    FILE    *ds9_afp;          /* file pointer to DS9 actor info file      */
    FILE    *tos_efp;          /* file pointer to TOS episode info file    */
    FILE    *tas_efp;          /* file pointer to TAS episode info file    */
    FILE    *tms_efp;          /* file pointer to TMS episode info file    */
    FILE    *tng_efp;          /* file pointer to TNG episode info file    */
    FILE    *ds9_efp;          /* file pointer to DS9 episode info file    */
    int     num_acts_tos;      /* number of actor records for TOS          */
    int     num_acts_tas;      /* number of actor records for TAS          */
    int     num_acts_tms;      /* number of actor records for TMS          */
    int     num_acts_tng;      /* number of actor records for TNG          */
    int     num_acts_ds9;      /* number of actor records for DS9          */
    int     num_eps_tos;       /* number of episode records for TOS        */
    int     num_eps_tas;       /* number of episode records for TAS        */
    int     num_eps_tms;       /* number of episode records for TMS        */
    int     num_eps_tng;       /* number of episode records for TNG        */
    int     num_eps_ds9;       /* number of episode records for DS9        */

    if (Acts ISNT NULL) {
        free (Acts);
        Acts = NULL;
    }
    if (Eps ISNT NULL) {
        free (Eps);
        Eps = NULL;
    }

    if ((tos_afp = fopen (TG_FILE_ACT_TOS, "r")) IS NULL)
        num_acts_tos = 0;
    else {
        fseek (tos_afp, 0L, SEEK_END);
        num_acts_tos = (int) (ftell (tos_afp) / ACTREC_LEN);
        fseek (tos_afp, 0L, SEEK_SET);
    }

    if ((tas_afp = fopen (TG_FILE_ACT_TAS, "r")) IS NULL)
        num_acts_tas = 0;
    else {
        fseek (tas_afp, 0L, SEEK_END);
        num_acts_tas = (int) (ftell (tas_afp) / ACTREC_LEN);
        fseek (tas_afp, 0L, SEEK_SET);
    }

    if ((tms_afp = fopen (TG_FILE_ACT_TMS, "r")) IS NULL)
        num_acts_tms = 0;
    else {
        fseek (tms_afp, 0L, SEEK_END);
        num_acts_tms = (int) (ftell (tms_afp) / ACTREC_LEN);
        fseek (tms_afp, 0L, SEEK_SET);
    }

    if ((tng_afp = fopen (TG_FILE_ACT_TNG, "r")) IS NULL)
        num_acts_tng = 0;
    else {
        fseek (tng_afp, 0L, SEEK_END);
        num_acts_tng = (int) (ftell (tng_afp) / ACTREC_LEN);
        fseek (tng_afp, 0L, SEEK_SET);
    }

    if ((ds9_afp = fopen (TG_FILE_ACT_DS9, "r")) IS NULL)
        num_acts_ds9 = 0;
    else {
        fseek (ds9_afp, 0L, SEEK_END);
        num_acts_ds9 = (int) (ftell (ds9_afp) / ACTREC_LEN);
        fseek (ds9_afp, 0L, SEEK_SET);
    }

    Num_Acts = num_acts_tos + num_acts_tas + num_acts_tms + num_acts_tng + num_acts_ds9;

    if ((Acts = (actrec) calloc (Num_Acts, ACTREC_LEN)) IS NULL) {
        fprintf (stderr, "\nNot enough memory for array of %d actor records (%d bytes each).\n", Num_Acts, ACTREC_LEN);
        fprintf (stderr, "\nEither wait until more memory is available, or run the program");
        fprintf (stderr, "\nwith the '+f' command line argument.\n\n");
        exit (-1);
    }

    tg_fill_acts (tos_afp, 0, num_acts_tos);
    tg_fill_acts (tas_afp, num_acts_tos, num_acts_tas);
    tg_fill_acts (tms_afp, num_acts_tos + num_acts_tas, num_acts_tms);
    tg_fill_acts (tng_afp, num_acts_tos + num_acts_tas + num_acts_tms, num_acts_tng);
    tg_fill_acts (ds9_afp, num_acts_tos + num_acts_tas + num_acts_tms + num_acts_tng, num_acts_ds9);

    if (tos_afp ISNT NULL)
        fclose (tos_afp);
    if (tas_afp ISNT NULL)
        fclose (tas_afp);
    if (tms_afp ISNT NULL)
        fclose (tms_afp);
    if (tng_afp ISNT NULL)
        fclose (tng_afp);
    if (ds9_afp ISNT NULL)
        fclose (ds9_afp);


    if ((tos_efp = fopen (TG_FILE_EPS_TOS, "r")) IS NULL)
        num_eps_tos = 0;
    else {
        fseek (tos_efp, 0L, SEEK_END);
        num_eps_tos = (int) (ftell (tos_efp) / EPSREC_LEN);
        fseek (tos_efp, 0L, SEEK_SET);
    }

    if ((tas_efp = fopen (TG_FILE_EPS_TAS, "r")) IS NULL)
        num_eps_tas = 0;
    else {
        fseek (tas_efp, 0L, SEEK_END);
        num_eps_tas = (int) (ftell (tas_efp) / EPSREC_LEN);
        fseek (tas_efp, 0L, SEEK_SET);
    }

    if ((tms_efp = fopen (TG_FILE_EPS_TMS, "r")) IS NULL)
        num_eps_tms = 0;
    else {
        fseek (tms_efp, 0L, SEEK_END);
        num_eps_tms = (int) (ftell (tms_efp) / EPSREC_LEN);
        fseek (tms_efp, 0L, SEEK_SET);
    }

    if ((tng_efp = fopen (TG_FILE_EPS_TNG, "r")) IS NULL)
        num_eps_tng = 0;
    else {
        fseek (tng_efp, 0L, SEEK_END);
        num_eps_tng = (int) (ftell (tng_efp) / EPSREC_LEN);
        fseek (tng_efp, 0L, SEEK_SET);
    }

    if ((ds9_efp = fopen (TG_FILE_EPS_DS9, "r")) IS NULL)
        num_eps_ds9 = 0;
    else {
        fseek (ds9_efp, 0L, SEEK_END);
        num_eps_ds9 = (int) (ftell (ds9_efp) / EPSREC_LEN);
        fseek (ds9_efp, 0L, SEEK_SET);
    }

    Num_Eps = num_eps_tos + num_eps_tas + num_eps_tms + num_eps_tng + num_eps_ds9;

    if ((Eps = (epsrec) calloc (Num_Eps, EPSREC_LEN)) IS NULL) {
        fprintf (stderr, "\nNot enough memory for array of %d episode records (%d bytes each).\n\n", Num_Eps, EPSREC_LEN);
        fprintf (stderr, "\nEither wait until more memory is available, or run the program");
        fprintf (stderr, "\nwith the '+f' command line argument.\n\n");
        free (Acts);
        exit (-1);
    }

    tg_fill_eps (tos_efp, 0, num_eps_tos);
    tg_fill_eps (tas_efp, num_eps_tos, num_eps_tas);
    tg_fill_eps (tms_efp, num_eps_tos + num_eps_tas, num_eps_tms);
    tg_fill_eps (tng_efp, num_eps_tos + num_eps_tas + num_eps_tms, num_eps_tng);
    tg_fill_eps (ds9_efp, num_eps_tos + num_eps_tas + num_eps_tms + num_eps_tng, num_eps_ds9);

    if (tos_efp ISNT NULL)
        fclose (tos_efp);
    if (tas_efp ISNT NULL)
        fclose (tas_efp);
    if (tms_efp ISNT NULL)
        fclose (tms_efp);
    if (tng_efp ISNT NULL)
        fclose (tng_efp);
    if (ds9_efp ISNT NULL)
        fclose (ds9_efp);
}


/* tg_fill_acts - Fills info from passed file pointer into specified spot
                  in Acts, filling in only specified number of records.
        Note: Acts must be appropriately calloc'ed, and Num_Acts must be
              correctly set, before calling this function.
*/

void tg_fill_acts (afp, start, num)
    FILE    *afp;               /* file pointer for actor info         */
    int     start;              /* starting location (index) in Acts   */
    int     num;                /* number of records to fill in        */
{
    int        i;               /* return value from fread()           */

    if ((num <= 0) OR (afp IS NULL))
        return;

    if ((start + num) > Num_Acts) {
        fprintf (stderr, "\nFatal Error: tg_fill_acts: start (%d) + num (%d) > Num_Acts (%d)!\n\n", start, num, Num_Acts);
        exit (-1);
    }

    if ((i = fread (&(Acts[start]), ACTREC_LEN, num, afp)) ISNT num) {
        fprintf (stderr, "\nFatal Error: tg_fill_acts: fread return (%d) not num (%d)!\n", i, num);
        exit (-1);
    }
}


/* tg_fill_eps - Fills info from passed file pointer into specified spot
                 in Eps, filling in only specified number of records.
        Note: Eps must be appropriately calloc'ed, and Num_Eps must be
              correctly set, before calling this function.
*/

void tg_fill_eps (efp, start, num)
    FILE    *efp;                 /* file pointer for episode info        */
    int     start;                /* starting location (index) in Eps     */
    int     num;                  /* number of records to fill in         */
{
    int     i;                    /* return value from fread()            */

    if ((num <= 0) OR (efp IS NULL))
        return;

    if ((start + num) > Num_Eps) {
        fprintf (stderr, "\nFatal Error: tg_fill_eps: start (%d) + num (%d) > Num_Eps (%d)!\n\n", start, num, Num_Eps);
        exit (-1);
    }

    if ((i = fread (&(Eps[start]), EPSREC_LEN, num, efp)) ISNT num) {
        fprintf (stderr, "\nFatal Error: tg_fill_eps: fread return (%d) not num (%d)!\n", i, num);
        exit (-1);
    }
}

